# Copyright 2012 Philip Chimento

"""Sound the system bell."""

# Import the toolkit-specific version
from __future__ import absolute_import

from .toolkit import toolkit_object
beep = toolkit_object('beep:beep')
