#------------------------------------------------------------------------------
# Copyright (c) 2009, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
#------------------------------------------------------------------------------

import logging

logger = logging.getLogger()
logger.warning('DEPRECATED: pyface.wx.clipboard, use pyface.api instead.')

from pyface.ui.wx.clipboard import Clipboard
clipboard = Clipboard()
